! This file is part of SusHi.
! 
! This file contains various routines related to the renormalization
! of the squark sectors and the determination of the bottom Yukawa
! coupling including resummation.
!
C-{{{ subroutine renormalize

      subroutine renormalize(fhflag,delmb,delAb)
      implicit none
      double precision tmp,b0fin,dAb(3),dmbfin,dAbos,
     &deltaML,alphas,alphaspdf,sthetaw,
     &dAbosfinmbHM,dAbosfintbHM,dAbosfinmbDS,dAbosfintbDS
      double precision dmbstd,dthetabstd,dAbstd
      double precision dmbnontanb, dmbtanb, mbMSDRtrans
      double precision dmbntb, dmbtb, dmbwosm, dmbntanbwosm
      double precision hbtree, db, dhb
      integer theint
      double precision Atr, thetatos
      double precision delmb, delAb, ifunc
      logical fhflag,expflag
      integer dep

      include 'commons/common-inputoutput.f'
      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      !determine whether to use DS (true) or RH (false) for sbottom cont.
      dsbflag = .true.
c      dsbflag = .false.

      !determine whether to use expanded CTs (true) or exact ones (false)
c      expflag = .true.
      expflag = .false.

      sthetaw = dsqrt(1.d0-(MW/MZ)**2)

!Various mb calculations

!Reconstruction of mb from FH
      if (fhflag.eqv.(.true.)) then
         mbsb = dsqrt(cthetab**2 * msb12 + sthetab**2 * msb22 - dMLb
     &- M3SQ**2 - MZ**2 * dcos(2.d0*beta) * (-1/2.d0+1/3.d0*sthetaw**2))
      end if

!Calculation of mb_DRbar_SM
      !at scale muR
      alphamuR = alphas(muR)
      mbDRSMmuR = mbMSDRtrans(mbMSbarmuR,1) !transition to DRbar
      !at scale mT
      alphamuR = alphas(mt)
      mbDRSMmt = mbMSDRtrans(mbMSbarmt,1)
      !at scale mbmb
      alphamuR = alphas(mbmb)
      mbDRSMmb = mbMSDRtrans(mbmb,1)
      !at scale muB
      alphamuR = alphas(muB)
      mbDRSMmuB = mbMSDRtrans(mbMSbarmuB,1)

!Calculation of the tree-level-sbottom-masses:
!      call calcsquarkmass(mbDRSMmt,0.d0) !effect of inserted mb small
      call calcsquarkmass(mbos,0.d0)

!Calculation of delta_b with tree-level-sbottom-masses:
!Setting of muD - decoupling scale supersymmetry
      if (model.gt.0) then
         muD = (mgl + msbot1 + msbot2)/3.d0
      else
         muD = muR
      end if
      !Calculation of mass m_b^MSbar(muD)
      call runmass(mbmb,alphaS(mbmb)/Pi,alphaS(muD)/Pi,nf,4,mbMSbarmuD)
      mbDRSMmuD = mbMSDRtrans(mbMSbarmuD,1)

      if (delmbfh.eq.1) then
      !Take delta_b from FH - includes ew corrections
         delta_mb = delmb
      !write(*,*) "delmb",delmb
      else if (delmbfh.eq.0) then
      !Calculate delta_b on our own
         alphamuR = alphas(muD)
         delmb = ifunc(msbot1**2,msbot2**2,mgl**2)/(2.d0*Pi)
         delAb = - cf*alphamuR*mgl*Ab*delmb
         delmb = cf*alphamuR*mgl*muSUSY*tanb*delmb
         delta_mb = delmb
      !write(*,*) "delmb",delmb
      end if

!Calculation of running bottom mass at scale muB
      mbDRMSmuB = mbMSbarmuB / (1.d0 + delmb)

!NB: alphamuR/Pi * dmbtb/mb gives the same as delmb above
!Calculation of mb_DRbar_MSSM in accordance to PS
      !at scale mT
      alphamuR = alphas(mt)
      dmbntanbwosm = dmbnontanb(.false.,.true.,mbDRSMmt,mt)
      dmbtb = dmbtanb(mbDRSMmt,mt)
      mbDRMSmt = mbDRSMmt
     & * (1.d0 + alphamuR/Pi * dmbntanbwosm/mbDRSMmt)
     & / (1.d0 - alphamuR/Pi * dmbtb/mbDRSMmt)
      !at scale muR
      alphamuR = alphas(muR)
      dmbntanbwosm = dmbnontanb(.false.,.true.,mbDRSMmuR,muR)
      dmbtb = dmbtanb(mbDRSMmuR,muR)
      mbDRMSmuR = mbDRSMmuR
     & * (1.d0 + alphamuR/Pi * dmbntanbwosm/mbDRSMmuR)
     & / (1.d0 - alphamuR/Pi * dmbtb/mbDRSMmuR)
      !at scale mbmb
      alphamuR = alphas(mbmb)
      dmbntanbwosm = dmbnontanb(.false.,.true.,mbDRSMmb,mbmb)
      dmbtb = dmbtanb(mbDRSMmb,mbmb)
      mbDRMSmb = mbDRSMmb
     & * (1.d0 + alphamuR/Pi * dmbntanbwosm/mbDRSMmb)
     & / (1.d0 - alphamuR/Pi * dmbtb/mbDRSMmb)

!mbOSsbot calculation similar to PS
!without any Yukawa corrections, just shifting mbDRMSmt to on-shell value
!      alphamuR = alphas(mt)
!      mbOSsbot = mbDRMSmt -dmbfin(.true.,.true.,mbDRMSmt,mt)*alphamuR/Pi

!      write(*,*) "mbsb1",msbot1
!      write(*,*) "mbsb2",msbot2
!      call calcsquarkmass(mbsb,dMLb)
!      write(*,*) "mbsb1",msbot1
!      write(*,*) "mbsb2",msbot2

!Calculation of the OS-sbottom-masses:
      if (fhflag.eqv.(.false.)) then
         alphamuR = alphas(muD)
         dMLb = alphamuR/Pi*deltaML(mt,mstop1,mstop2,cthetat,sthetat,
     &        mbos,msbot1,msbot2,cthetab,sthetab,mgl,muD) 
         mb = mbos
         mb2 = mb**2
         mt2 = mt**2
         call sbottomrenormHM(thetabos,dmbsbos,dthetabos
     &        ,dmsb1,dmsb2,muD)
         mbsb = mbDRSMmuD - ( 2.d0*mb/(msb12-msb22)
     &        * (dmsb1(1)*msb12 - dmsb2(1)*msb22)
     &        + 2.d0/t2thetab*mb * dthetabos(1)
     &        - 2.d0*mb2/(msb12-msb22)/s2thetab
     &        * dAbosfinmbHM(dthetabos(1),muD)) * alphamuR/Pi
      endif

      call calcsquarkmass(mbsb,dMLb)

!Various mb MSbar and DRbar values:
      write(*,105)
      write(*,100) 'Various internal (s)bottom masses/parameters:'
      write(*,101) 'm_b^MSbar(mb):         ',mbmb
      write(*,101) 'm_b^MSbar(muR):        ',mbMSbarmuR
      write(*,101) 'm_b^OS:                ',mbOS
      if (fhflag.eqv.(.true.)) then
         write(*,101) 'm_b from FeynHiggs:    ',mbsb
      else
         write(*,101) 'm_b dependent sbottom: ',mbOSsbot
      end if
      write(*,102) 'Delta_b:              ',delmb
      write(*,105)
!End of mb/sbottom calculations

!Calculation of counterterms in mbsb, Ab and thetab
      mb = mbsb
      mb2 = mb**2

      !vll ueberfluessig:
c      Ab = (msb12-msb22)/2/mb*s2thetab+muSUSY*tanb

      alphamuR = alphasPDF(muR)

      if(expflag.eqv..true.) then
         call sbottomrenormDS(mbsb,thetabos,dmbsbos,dthetabos
     &        ,dmsb1,dmsb2,muR)
      else
         call sbottomrenormHM(thetabos,dmbsbos,dthetabos
     &        ,dmsb1,dmsb2,muR)
      endif

      dep = 3
      dAb = 0.d0
      if(mbsbch.eq.0) then      !mb OS
         dmbsb = dmbsbos
      else if(mbsbch.eq.1) then !mb DRbar
         dmbsb = 0.d0
      else
         dep = 0
      endif
      if(thetabch.eq.0) then    !thetab OS
         dthetab(1) = 0.d0
         dthetab(2) = dthetabos(1)
         dthetab(3) = dthetabos(2)
      else if(thetabch.eq.1) then !thetab DRbar
         dthetab = 0.d0
      else
         if(dep.eq.0) then
            write(*,*) 'error: two dependent quantities.'
            stop
         endif
         dep = 1
      endif
      if(Abch.eq.0) then        !Ab OS
         if(dep.eq.0) then
            if(expflag.eqv..true.) then
               dAb(3) = dAbosfintbDS(dthetab(2),muR)
            else
               dAb(3) = dAbosfintbHM(dthetab(2),muR)
            endif
         else if(dep.eq.1) then
            if(expflag.eqv..true.) then
               dAb(3) = dAbosfinmbDS(dmbsb(2),muR)
            else
               dAb(3) = dAbosfinmbHM(dmbsb(2),muR)
            endif
         else
            write(*,*) 'error: no dependent quantity.'
            stop
         endif
      else if(Abch.eq.1) then   !Ab DRbar
         dAb(3) = 0.d0
      else
         if(dep.lt.2) then
            write(*,*) 'error: two dependent quantities.'
            stop
         endif
         dep = 2
      endif

      if(dep.eq.0) then
      !mb dependent
         dmbsb(1) = 0.d0
         dmbsb(2) = 2.d0*mb/(msb12-msb22)
     &        * (dmsb1(1)*msb12 - dmsb2(1)*msb22)
     &        + 2.d0/t2thetab*mb * dthetab(2)
         if(expflag.eqv..false.) then
            dmbsb(2) = dmbsb(2) - 2.d0*mb2/(msb12-msb22)/s2thetab*dAb(3)
         endif
         dmbsb(3) = 0.d0
      else if(dep.eq.1) then
      !thetab dependent
      !not offered as option
         write(*,*) 'error: theta_b cannot be chosen',
     &' as a dependent quantity. '
         stop
         dthetab(1) = t2thetab/mb/2.d0 * dmbsb(1)
         dthetab(2) = -t2thetab/(msb12-msb22)
     &        * (dmsb1(1)*msb12 - dmsb2(1)*msb22)
     &        + t2thetab/mb/2.d0 * dmbsb(2)
         if(expflag.eqv..false.) then
            dthetab(2) = dthetab(2) + mb/(msb12-msb22)/c2thetab * dAb(3)
         else
            dthetab(3) = 0.d0
         endif
      else if(dep.eq.2) then
      !Ab dependent

         dAb(1) = - s2thetab*(msb12-msb22)/2.d0/mb2 * dmbsb(1)

         dAb(2) = s2thetab/mb
     &        * (dmsb1(1)*msb12 - dmsb2(1)*msb22)
     &        - s2thetab*(msb12-msb22)/2.d0/mb2 * dmbsb(2)
     &        + c2thetab*(msb12-msb22)/mb * dthetab(2)

         dAb(3) = s2thetab/mb
     &        * (dmsb1(2)*msb12 - dmsb2(2)*msb22)
     &        - s2thetab*(msb12-msb22)/2.d0/mb2 * dmbsb(3)
     &        + c2thetab*(msb12-msb22)/mb * dthetab(3)

      else
         write(*,*) 'error: no dependent quantity.'
         stop
      endif

      dthetabstd = dthetabos(1)
      dmbstd = 2.d0*mb/(msb12-msb22)
     &        * (dmsb1(1)*msb12 - dmsb2(1)*msb22)
     &        + 2.d0/t2thetab*mb * dthetabstd

      if(expflag.eqv..true.) then
         dAbstd = dAbosfintbDS(dthetabstd,muR)
      else
         dAbstd = dAbosfintbHM(dthetabstd,muR)
         dmbstd = dmbstd - 2.d0*mb2/(msb12-msb22)/s2thetab * dAbstd
      endif

c$$$      write(*,*) mgl,Ab,msbot1,msbot2,mh,mb,thetabos,muSUSY,beta,
c$$$     &     dasin(sthetaw),mz,alpha,muR
c$$$      write(*,*) 'mb_OS: ',dmbsbos(1),dmbsbos(1)+dmbsbos(2),
c$$$     &     dmbsbos(1)+dmbsbos(2)+dmbsbos(3)
c$$$      write(*,*) 'tb_OS: ',dthetabos(1),dthetabos(1)+dthetabos(2)
c$$$      write(*,*) 'msb_OS: ',dmsb1(1),dmsb1(1)+dmsb1(2),
c$$$     &     dmsb2(1),dmsb2(1)+dmsb2(2)
c$$$      write(*,*) 'mb_RS: ',dmbsb(1),dmbsb(1)+dmbsb(2),
c$$$     &     dmbsb(1)+dmbsb(2)+dmbsb(3)
c$$$      write(*,*) 'tb_RS: ',dthetab(1),dthetab(1)+dthetab(2),
c$$$     &     dthetab(1)+dthetab(2)+dthetab(3)
c$$$      write(*,*) 'Ab_RS: ',dAb(1),dAb(1)+dAb(2),dAb(1)+dAb(2)+dAb(3)
c$$$      write(*,*) 'delstd:',dmbstd, dthetabstd, dAbstd

!CT in Ab and thetab
      Ab = Ab + alphamuR/Pi * (dAbstd - (dAb(1)+dAb(2)+dAb(3)))
      thetab = thetabos+alphamuR/Pi*(dthetabstd-(dthetab(1)+dthetab(2)))
      mbsb = mbsb + alphamuR/Pi * (dmbstd - (dmbsb(1)+dmbsb(2)))
      mbsb2 = mbsb**2
      dmbsb = dmbsb / mbsb

!Settings for internal bottom mass/Yukawa coupling
      call sbottomrenormDS(mbos,thetabos,dmbsbos,dthetabos
     &        ,dmsb1,dmsb2,muR)

      if(mbrunloop.eq.0) then
      !mb_OS:
         mb = mbos
         dmb(1) = 0.d0
         dmb(2) = 0.d0
         dmb(3) = 0.d0
      elseif(mbrunloop.eq.1) then
      !mb_DRbar_MSSM(muR):
         mb = mbDRMSmuR
         dmb(1) = dmbsbos(1)
         dmb(3) = -mbos*(5/3.d0+dlog(muR**2/mbos**2))
         dmb(2) = dmbsbos(2)-dmb(3)
c$$$         tmp = dmbfin(.true.,.true.,mbsb,muR)
c$$$         write(*,*) tmp,dmb(1)+dmb(2)+dmb(3)
c$$$         tmp = dmbfin(.false.,.true.,mbsb,muR)
c$$$         write(*,*) tmp,dmb(1)+dmb(2)
c$$$         stop
      elseif(mbrunloop.eq.2) then
      !mb_DRbar_MSSM(mb):
         mb = mbDRMSmb
         dmb(1) = dmbsbos(1)
         dmb(3) = -mbos*(5/3.d0+dlog(mbmb**2/mbos**2))
         dmb(2) = dmbsbos(2) + mbos*2/3.d0*dlog(muR**2/mbmb**2)-dmb(3)
c$$$         tmp = dmbfin(.true.,.true.,mbsb,mbmb)
c$$$         write(*,*) tmp,dmb(1)+dmb(2)+dmb(3)
c$$$         tmp = dmbfin(.false.,.true.,mbsb,mbmb)
c$$$         write(*,*) tmp,dmb(1)+dmb(2)
c$$$         stop
      elseif(mbrunloop.eq.3) then
      !mb_DRbar_MSSM(mub):
         mb = mbDRMSmub
         dmb(1) = dmbsbos(1)
         dmb(3) = -mbos*(5/3.d0+dlog(mub**2/mbos**2))
         dmb(2) = dmbsbos(2) + mbos*2/3.d0*dlog(muR**2/mub**2)-dmb(3)
      else
         stop
      endif
      mb2 = mb**2
      dmb = dmb / mbos

      mbyuk = mbos
      dgb = 0.d0
      !NOTE: dgb only contains the SM part, the SUSY part is in virtds.
      !The sign of dgb is different from the sign in virtds.
      if ((mbrunyuk.eq.1).or.(mbrunyuk.eq.2)) then
      !difference between 1 and 2: 1 muB=mbmb, 2 muB = muR
         mbyuk = mbDRMSmuB
         dgb = 2.d0 * dlog(muD/muB)
     &   + 2.d0 * dlog(mb/muD) - 5/3.d0 + 1/3.d0
!          dgb = 2.d0 * dlog(muD/muB) * alphasPDF(muB)/alphasPDF(muR)
!     &   + 2.d0 * dlog(mb/muD) - 5/3.d0
!     &   + 1/3.d0 * alphasPDF(muB)/alphasPDF(muR)
         gb = mbDRMSmuB / mbos * yukfac(3) 
         tanbresum = 0
      else if(mbrunyuk.eq.3) then
         mbyuk = mbDRMSmuR
         dgb = (dmbfin(.true.,.true.,mbos,muR)
     &        - dmbfin(.false.,.true.,mbos,muR))
     & / mbDRMSmuR * mbDRMSmuR / mbos !last factor: Kfac
         gb = mbDRMSmuR / mbos * yukfac(3) 
         tanbresum = 0
      else if(mbrunyuk.eq.4) then
         mbyuk = mbDRMSmb
         dgb = (dmbfin(.true.,.true.,mbos,mbmb) 
     &        - dmbfin(.false.,.true.,mbos,mbmb))
     & / mbDRMSmb * mbDRMSmb / mbos !last factor: Kfac
         gb = mbDRMSmb / mbos * yukfac(3) 
         tanbresum = 0
      else
      !Delta_b resummation:
         if(tanbresum.eq.1) then
            gb = yukfac(3) / ( 1.d0 + delmb)
         else if(tanbresum.eq.2) then   
            if (pseudo.eq.1) then
               gb = yukfac(3) * (1.d0 - (1.d0/tanb**2)*delmb)
     &              * (1.d0/(1.d0+delmb))
            else if (pseudo.eq.0) then
               gb = yukfac(3) * (1.d0 - (1.d0/(tanb*Tan(alpha)))*delmb)
     &              * (1.d0/(1.d0+delmb))
            else if (pseudo.eq.2) then
               gb = yukfac(3) * (1.d0 + (Tan(alpha)/tanb)*delmb)
     &              * (1.d0/(1.d0+delmb))
            end if 
         else
            gb = yukfac(3)
         end if
      end if

      !Final settings of renormalize
      cthetab = dcos(thetab)
      sthetab = dsin(thetab)
      t2thetab = dtan(2*thetab)
      c2thetab = cthetab**2 - sthetab**2
      s2thetab = 2*sthetab*cthetab
      c4thetab = c2thetab**2-s2thetab**2

!Screen output of relevant data in the (s)bottom sector:
      write(*,103) 'Bottom masses used for:              '
      write(*,101) 'internal masses:      ',mb
      if (mbrunyuk.gt.0) then
      write(*,101) 'bottom Yuk:           ',mbyuk
      else
      write(*,101) 'bottom Yuk:           ',mbyuk*gb/yukfac(3)
      end if
      write(*,101) 'sbottom sector:       ',mbsb
      write(*,105)
      write(*,103) 'Renormalized sbottom sector:         '
      write(*,101) 'm_b:                   ',mbsb
      write(*,102) 'sin(theta_b):          ',sin(thetab)
      write(*,102) 'cos(theta_b):          ',cos(thetab)
      write(*,101) 'A_b:                   ',Ab
      write(*,101) 'OS-sbottom mass 1:    ',msbot1
      write(*,101) 'OS-sbottom mass 2:    ',msbot2
      write(*,105)

!Recalculation of counterterms using the new parameters
      call sbottomrenormDS(mbsb,thetab,dmbsbos,dthetabos
     &        ,dmsb1,dmsb2,muR)

      dep = 3
      dAb = 0.d0
      if(mbsbch.eq.0) then      !mb OS
         dmbsb = dmbsbos
      else if(mbsbch.eq.1) then !mb DRbar
         dmbsb = 0.d0
      else
         dep = 0
      endif
      if(thetabch.eq.0) then    !thetab OS
         dthetab(1) = 0.d0
         dthetab(2) = dthetabos(1)
         dthetab(3) = dthetabos(2)
      else if(thetabch.eq.1) then !thetab DRbar
         dthetab = 0.d0
      else
         dep = 1
      endif
      if(Abch.eq.0) then        !Ab OS
         if(dep.eq.0) then
            dAb(3) = dAbosfintbDS(dthetab(2),muR)
         else if(dep.eq.1) then
            dAb(3) = dAbosfinmbDS(dmbsb(2),muR)
         endif
      else if(Abch.eq.1) then   !Ab DRbar
         dAb(3) = 0.d0
      else
         dep = 2
      endif

      if(dep.eq.0) then
!     mb dependent
         dmbsb(1) = 0.d0
         dmbsb(2) = 2.d0*mbsb/(msb12-msb22)
     &        * (dmsb1(1)*msb12 - dmsb2(1)*msb22)
     &        + 2.d0/t2thetab*mbsb * dthetab(2)
         dmbsb(3) = 0.d0
      else if(dep.eq.1) then
!     thetab dependent
         dthetab(1) = t2thetab/mbsb/2.d0 * dmbsb(1)
         dthetab(2) = -t2thetab/(msb12-msb22)
     &        * (dmsb1(1)*msb12 - dmsb2(1)*msb22)
     &        + t2thetab/mbsb/2.d0 * dmbsb(2)
         dthetab(3) = 0.d0
      else if(dep.eq.2) then
!     Ab dependent

         dAb(1) = - s2thetab*(msb12-msb22)/2.d0/mbsb2 * dmbsb(1)

         dAb(2) = s2thetab/mbsb
     &        * (dmsb1(1)*msb12 - dmsb2(1)*msb22)
     &        - s2thetab*(msb12-msb22)/2.d0/mbsb2 * dmbsb(2)
     &        + c2thetab*(msb12-msb22)/mbsb * dthetab(2)

         dAb(3) = s2thetab/mbsb
     &        * (dmsb1(2)*msb12 - dmsb2(2)*msb22)
     &        - s2thetab*(msb12-msb22)/2.d0/mbsb2 * dmbsb(3)
     &        + c2thetab*(msb12-msb22)/mbsb * dthetab(3)
      endif

      dAbds = dAb
      dmbsb = dmbsb / mbsb

 100  format(a45)
 103  format(a30)
 101  format(a22,'    ',f10.5, '   GeV')
 102  format(a22,'    ',f10.5)
 105  format('#--------------------------------------------------#')
      end

C-}}}
C-{{{ subroutine calcsquarkmass

      subroutine calcsquarkmass(mb_in,dMLb_in)
      implicit none
      double precision tmp, sthetaW, mb_in, dMLb_in
      double precision MsbotEig(2,2), MstopEig(2,2)
      double precision StopMix(2,2), SbotMix(2,2)
      double precision TestMatrix(2,2)

      include 'commons/common-inputoutput.f'
      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      sthetaw = dsqrt(1.d0-(MW/MZ)**2)
      mb = mb_in
      mb2 = mb**2
      mt2 = mt**2

      MstopEig(1,1) = M3SQ**2 + mt2 + MZ**2 
     &* dcos(2.d0*beta) * (1/2.d0-2/3.d0*sthetaw**2)
      MstopEig(2,2) = M3SU**2 + mt2 + MZ**2 
     &* dcos(2.d0*beta) * (2/3.d0*sthetaw**2)
      MstopEig(1,2) = mt * (At - muSUSY/tanb)
      MstopEig(2,1) = MstopEig(1,2)

      !calculate mstop1 and mstop2 with mstop1 < mstop2
      mstop1=dsqrt(M3SQ**2/2.d0 + M3SU**2/2.d0 + mt2
     &       +MZ**2/4.d0*dcos(2.d0*beta)
     &     -dsqrt((M3SQ**2 - M3SU**2 + MZ**2*dcos(2.d0*beta)
     &       *(1/2.d0-4/3.d0*sthetaw**2))**2
     &     + 4.d0*mt2*(At-muSUSY/tanb)**2 )/2.d0)
      mstop2=dsqrt(M3SQ**2/2.d0 + M3SU**2/2.d0 + mt2
     &       +MZ**2/4.d0*dcos(2.d0*beta)
     &     +dsqrt((M3SQ**2 - M3SU**2 + MZ**2 *dcos(2.d0*beta)
     &       *(1/2.d0-4/3.d0*sthetaw**2))**2
     &     + 4.d0*mt2*(At-muSUSY/tanb)**2 )/2.d0)
      mst12 = mstop1**2
      mst22 = mstop2**2

      thetat = dasin(2.d0*mt/(mst12-mst22)*(At-muSUSY/tanb))/2.d0
      sthetat = dsin(thetat)
      cthetat = dcos(thetat)

      !change sin <-> cos
      if(dabs(cthetat**2 *mst12 +sthetat**2 *mst22 
     & - MstopEig(1,1)).Gt.(1.e-3)) then
      thetat = Pi/2.d0 - thetat
      end if

      sthetat = dsin(thetat)
      cthetat = dcos(thetat)

      c2thetat = cthetat**2 - sthetat**2
      s2thetat = 2*sthetat*cthetat

!      write(*,*) "---------------------------Test diagonalization"
!      write(*,*) "thetat",thetat,sthetat,cthetat
!      write(*,*) "11",cthetat**2 * mst12 + sthetat**2 * mst22
!     & ,MstopEig(1,1)
!      write(*,*) "12",cthetat*sthetat* (mst12 - mst22)
!     & ,MstopEig(1,2)
!      write(*,*) "22",sthetat**2 * mst12 + cthetat**2 * mst22
!     & ,MstopEig(2,2)
!      StopMix(1,1) = cthetat
!      StopMix(1,2) = sthetat
!      StopMix(2,1) = -sthetat
!      StopMix(2,2) = cthetat
!      TestMatrix = Matmul(Matmul(StopMix,MstopEig),Transpose(StopMix))
!      write(*,*) "Matrix-1",Sqrt(TestMatrix(1,1)),TestMatrix(1,2)
!      write(*,*) "Matrix-2",TestMatrix(2,1),Sqrt(TestMatrix(2,2))
!      write(*,*) "Masses",mstop1,mstop2

!      MsbotEig(1,1) = M3SQ**2 + dMLb_in + mb2 + MZ**2 
!     &* dcos(2.d0*beta) * (-1/2.d0+1/3.d0*sthetaw**2)
!      MsbotEig(2,2) = M3SD**2 + mb2 + MZ**2 
!     &* dcos(2.d0*beta) * (-1/3.d0*sthetaw**2)
!      MsbotEig(1,2) = mb * (Ab - muSUSY*tanb)
!      MsbotEig(2,1) = MsbotEig(1,2)

      !calculate msbot1 and msbot2 with msbot1 < msbot2
      msbot1=dsqrt(M3SQ**2/2.d0 + M3SD**2/2.d0 + dMLb_in/2.d0 + mb2
     &     -MZ**2/4.d0*dcos(2*beta)-dsqrt(
     &     (M3SQ**2 - M3SD**2 + dMLb_in
     &     + MZ**2*dcos(2*beta)*(-1/2.d0+2/3.d0*sthetaw**2))**2
     &     + 4.d0*mb2*(Ab-muSUSY*tanb)**2 )/2.d0)
      msbot2=dsqrt(M3SQ**2/2.d0 + M3SD**2/2.d0 + dMLb_in/2.d0 + mb2
     &     -MZ**2/4.d0*dcos(2*beta)+dsqrt(
     &     (M3SQ**2 - M3SD**2 + dMLb_in
     &     + MZ**2*dcos(2*beta)*(-1/2.d0+2/3.d0*sthetaw**2))**2
     &     + 4.d0*mb2*(Ab-muSUSY*tanb)**2 )/2.d0)

      msb12 = msbot1**2
      msb22 = msbot2**2

      thetabos = dasin(2.d0*mb/(msb12-msb22)*(Ab-muSUSY*tanb))/2.d0

      sthetab = dsin(thetabos)
      cthetab = dcos(thetabos)

      if(dabs(cthetab**2 * msb12 + sthetab**2 * msb22 
     & - MsbotEig(1,1)).Gt.(1.e-3)) then
         thetabos = Pi/2.d0 - thetabos
      end if

      sthetab = dsin(thetabos)
      cthetab = dcos(thetabos)

      thetab = thetabos
      t2thetab = dtan(2*thetab)
      c2thetab = cthetab**2 - sthetab**2
      s2thetab = 2*sthetab*cthetab
      c4thetab = c2thetab**2-s2thetab**2

!      write(*,*) "---------------------------Test diagonalization"
!      write(*,*) "thetab",thetabos,sthetab,cthetab
!      write(*,*) "11",cthetab**2 * msb12 + sthetab**2 * msb22
!     & ,MsbotEig(1,1)
!      write(*,*) "12",cthetab*sthetab* (msb12 - msb22)
!     & ,MsbotEig(1,2)
!      write(*,*) "22",sthetab**2 * msb12 + cthetab**2 * msb22
!     & ,MsbotEig(2,2)
!      SbotMix(1,1) = cthetab
!      SbotMix(1,2) = sthetab
!      SbotMix(2,1) = -sthetab
!      SbotMix(2,2) = cthetab
!      TestMatrix = Matmul(Matmul(SbotMix,MsbotEig),Transpose(SbotMix))
!      write(*,*) "Matrix-1",Sqrt(TestMatrix(1,1)),TestMatrix(1,2)
!      write(*,*) "Matrix-2",TestMatrix(2,1),Sqrt(TestMatrix(2,2))
!      write(*,*) "Masses",msbot1,msbot2

      end

C-}}}
C-{{{ function deltaML

      function deltaML(mtop,mstop1,mstop2,cthetat,sthetat,mb,msb1,
     &msb2,cthetab,sthetab,mgluino,mu)
      implicit none
      double precision mtop,mstop1,mstop2,cthetat,mb,msb1,
     &msb2,cthetab,mgluino,lb,lb1,lb2,lt,lt1,lt2,lg,b0fin,
     &mu,sthetab,sthetat,c2thetab,s2thetab,c2thetat,s2thetat
     &,M,deltaML
c      double complex b0finim

      c2thetat = cthetat**2 - sthetat**2
      s2thetat = 2*sthetat*cthetat
      c2thetab = cthetab**2 - sthetab**2
      s2thetab = 2*sthetab*cthetab

      lb = 2.d0*dlog(mu/mb)
      lb1 = 2.d0*dlog(mu/msb1)
      lb2 = 2.d0*dlog(mu/msb2)
      lt = 2.d0*dlog(mu/mtop)
      lt1 = 2.d0*dlog(mu/mstop1)
      lt2 = 2.d0*dlog(mu/mstop2)
      lg = 2.d0*dlog(mu/mgluino)

      deltaML =(-8*mb**2 - 4*lb*mb**2 + 4*msb1**2 + 3*c2thetab*msb1**2 + 
     -     2*lb1*msb1**2 + c2thetab*lb1*msb1**2 + 4*msb2**2 - 
     -     3*c2thetab*msb2**2 + 2*lb2*msb2**2 - c2thetab*lb2*msb2**2 - 
     -     4*mstop1**2 - 3*c2thetat*mstop1**2 - 2*lt1*mstop1**2 - 
     -     c2thetat*lt1*mstop1**2 - 4*mstop2**2 + 3*c2thetat*mstop2**2 - 
     -     2*lt2*mstop2**2 + c2thetat*lt2*mstop2**2 + 8*mtop**2 + 
     -     4*lt*mtop**2 + (mb**2 + mgluino**2 - msb1**2 - 
     -     2*mb*mgluino*s2thetab)*b0fin(mb**2,mgluino,msb1,mu) + 
     -     (mb**2 + mgluino**2 - msb2**2 + 2*mb*mgluino*s2thetab)*
     -     b0fin(mb**2,mgluino,msb2,mu) + 
     -     mb**2*b0fin(msb1**2,mb,mgluino,mu) + 
     -     c2thetab*mb**2*b0fin(msb1**2,mb,mgluino,mu) + 
     -     mgluino**2*b0fin(msb1**2,mb,mgluino,mu) + 
     -     c2thetab*mgluino**2*b0fin(msb1**2,mb,mgluino,mu) - 
     -     msb1**2*b0fin(msb1**2,mb,mgluino,mu) - 
     -     c2thetab*msb1**2*b0fin(msb1**2,mb,mgluino,mu) - 
     -     2*mb*mgluino*s2thetab*b0fin(msb1**2,mb,mgluino,mu) + 
     -     mb**2*b0fin(msb2**2,mb,mgluino,mu) - 
     -     c2thetab*mb**2*b0fin(msb2**2,mb,mgluino,mu) + 
     -     mgluino**2*b0fin(msb2**2,mb,mgluino,mu) - 
     -     c2thetab*mgluino**2*b0fin(msb2**2,mb,mgluino,mu) - 
     -     msb2**2*b0fin(msb2**2,mb,mgluino,mu) + 
     -     c2thetab*msb2**2*b0fin(msb2**2,mb,mgluino,mu) + 
     -     2*mb*mgluino*s2thetab*b0fin(msb2**2,mb,mgluino,mu) - 
     -     mgluino**2*b0fin(mstop1**2,mtop,mgluino,mu) - 
     -     c2thetat*mgluino**2*b0fin(mstop1**2,mtop,mgluino,mu) + 
     -     mstop1**2*b0fin(mstop1**2,mtop,mgluino,mu) + 
     -     c2thetat*mstop1**2*b0fin(mstop1**2,mtop,mgluino,mu) - 
     -     mtop**2*b0fin(mstop1**2,mtop,mgluino,mu) - 
     -     c2thetat*mtop**2*b0fin(mstop1**2,mtop,mgluino,mu) + 
     -     2*mgluino*mtop*s2thetat*b0fin(mstop1**2,mtop,mgluino,mu) - 
     -     mgluino**2*b0fin(mstop2**2,mtop,mgluino,mu) + 
     -     c2thetat*mgluino**2*b0fin(mstop2**2,mtop,mgluino,mu) + 
     -     mstop2**2*b0fin(mstop2**2,mtop,mgluino,mu) - 
     -     c2thetat*mstop2**2*b0fin(mstop2**2,mtop,mgluino,mu) - 
     -     mtop**2*b0fin(mstop2**2,mtop,mgluino,mu) + 
     -     c2thetat*mtop**2*b0fin(mstop2**2,mtop,mgluino,mu) - 
     -     2*mgluino*mtop*s2thetat*b0fin(mstop2**2,mtop,mgluino,mu) - 
     -     mgluino**2*b0fin(mtop**2,mgluino,mstop1,mu) + 
     -     mstop1**2*b0fin(mtop**2,mgluino,mstop1,mu) - 
     -     mtop**2*b0fin(mtop**2,mgluino,mstop1,mu) + 
     -     2*mgluino*mtop*s2thetat*
     -     b0fin(mtop**2,mgluino,mstop1,mu) - 
     -     mgluino**2*b0fin(mtop**2,mgluino,mstop2,mu) + 
     -     mstop2**2*b0fin(mtop**2,mgluino,mstop2,mu) - 
     -     mtop**2*b0fin(mtop**2,mgluino,mstop2,mu) - 
     -     2*mgluino*mtop*s2thetat*b0fin(mtop**2,mgluino,mstop2,mu)
     -     )/3.d0
      end

C-}}}

C-{{{ sbottomrenormHM

      subroutine sbottomrenormHM(thetab_in,dhbhb_out,
     &dthetab_out,dmsb1_out,dmsb2_out,mu_in)
      implicit none
      double precision dhbhb_out(3), dthetab_out(2)
      double precision thetab_in, dmsb1_out(2), dmsb2_out(2), mu_in
      double precision dmsb1osfin,dmsb2osfin,dthetabosfin,
     &dmbfin,tmp

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      lb = dlog(mu_in**2/mb2)
      lb1 = dlog(mu_in**2/msb12)
      lb2 = dlog(mu_in**2/msb22)
      lt = dlog(mu_in**2/mt2)
      lt1 = dlog(mu_in**2/mst12)
      lt2 = dlog(mu_in**2/mst22)
      lg = 2*dlog(mu_in/mgl)

      dmsb1_out(1) = dmsb1osfin()/msbot1
      dmsb1_out(2) = 0.d0
      dmsb2_out(1) = dmsb2osfin()/msbot2
      dmsb2_out(2) = 0.d0
      dthetab_out(1) = dthetabosfin()
      dthetab_out(2) = 0.d0
      dhbhb_out(1) = 0.d0
      dhbhb_out(2) = dmbfin(.true.,.true.,mbsb,mu_in)
      dhbhb_out(3) = 0.d0
      end

C-}}}
C-{{{ sbottomrenormDS

      subroutine sbottomrenormDS(mb_in,thetab_in,dhbhb_out,
     &dthetab_out,dmsb1_out,dmsb2_out,mu_in)
      !formulas from 1007.3465 (B1)-(B4): Degrassi, Slavich
      implicit none
      double precision dhbhb_out(3), dthetab_out(2)
      double precision thetab_in, dmsb1_out(2), dmsb2_out(2), mu_in
      double precision mb_in, mb2_in
      double precision cthetab2_in, sthetab2_in,x1,x2

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      cthetab2_in = dcos(2.d0*thetab_in)
      sthetab2_in = dsin(2.d0*thetab_in)
      mb2_in = mb_in**2

      dmsb1_out(1) = msb12 * 1/3.d0 *
     & (3.d0 * dlog(msb12/mu_in**2) 
     & - 3.d0 - cthetab2_in**2 * (dlog(msb12/mu_in**2) - 1.d0)
     & - sthetab2_in**2 * msb22/msb12 * (dlog(msb22/mu_in**2) - 1.d0)
     & - 6.d0 * mgl**2/msb12
     & - 4.d0 * (1.d0 - 2.d0 * mgl**2/msb12)*dlog(mgl/mu_in)
     & - 2.d0 * (1.d0 - mgl**2/msb12)**2
     &   * dlog(dabs(1.d0 - msb12/mgl**2))  )

      dmsb1_out(2) = (-4*sthetab2_in*mgl
     -     *(-mgl**2*dlog(dabs(1.d0 - msb12/mgl**2)) + 
     -       msb12*(-2 + dlog(dabs(mgl**2 - msb12)/mu_in**2)))*
     -     dsqrt(mb2_in))/(3.d0*msb12)

      dmsb2_out(1) = msb22 * 1/3.d0 *
     & (3.d0 * dlog(msb22/mu_in**2) 
     & - 3.d0 - cthetab2_in**2 * (dlog(msb22/mu_in**2) - 1.d0)
     & - sthetab2_in**2 * msb12/msb22 * (dlog(msb12/mu_in**2) - 1.d0)
     & - 6.d0 * mgl**2/msb22
     & - 4.d0 * (1.d0 - 2.d0 * mgl**2/msb22)*dlog(mgl/mu_in)
     & - 2.d0 * (1.d0 - mgl**2/msb22)**2
     &   * dlog(dabs(1.d0 - msb22/mgl**2))  )

      dmsb2_out(2) = (4*sthetab2_in*mgl*
     -     (-mgl**2*dlog(dabs(1.d0 - msb22/mgl**2)) + 
     -       msb22*(-2 + dlog(dabs(mgl**2 - msb22)/mu_in**2)))*
     -     dsqrt(mb2_in))/(3.d0*msb22)

      dthetab_out(1) = sthetab2_in * 1/3.d0 *
     & (-2.d0 * cthetab2_in**2 + 2.d0 * cthetab2_in**2 /(msb12 - msb22)
     &  * (msb12 * dlog(msb12/mu_in**2) - msb22 * dlog(msb22/mu_in**2)))

      dthetab_out(2) = (4*mgl*cthetab2_in**2
     -     *(mgl**2*msb22*dlog(dabs(1.d0 - msb12/mgl**2)) + 
     -      msb12*(mgl**2*dlog(dabs(1.d0 - msb22/mgl**2)) + 
     -      msb22*(4 - dlog(dabs((mgl**2 - msb12)*(mgl**2 - msb22))/
     -     mu_in**4))))*dsqrt(mb2_in))/
     -     (3.d0*msb12*(msb12 - msb22)*msb22)

      x1 = msb12/mgl**2
      x2 = msb22/mgl**2

!     dhbhb_pole from (32) and (29)
      dhbhb_out(1) = (sthetab2_in*mgl*((x1*dlog(x1))/(-1 + x1) - 
     -     (x2*dlog(x2))/(-1 + x2)))/(3.*dsqrt(mb2_in))
      dhbhb_out(2) = dlog(mb2_in/mu_in**2) + 
     -   (-5 + (-3 + x1)/(4.*(-1 + x1)) + (-3 + x2)/(4.*(-1 + x2)) - 
     -      dlog(mgl**2/mu_in**2) - 
     -      ((-2 + x1)*x1*dlog(x1))/(2.*(-1 + x1)**2) - 
     -      ((-2 + x2)*x2*dlog(x2))/(2.*(-1 + x2)**2))/3.d0
      dhbhb_out(3) = sthetab2_in * (
     -     ((-3 + x1)*x1)/(2.*(-1 + x1)**2) - 
     -     ((-3 + x2)*x2)/(2.*(-1 + x2)**2) + 
     -     (x1*dlog(x1))/(-1 + x1)**3 - 
     -     (x2*dlog(x2))/(-1 + x2)**3 ) * dsqrt(mb2_in)/(3.d0*mgl)


      dhbhb_out = dhbhb_out*mb_in
      dmsb1_out = dmsb1_out/2.d0/msb12
      dmsb2_out = dmsb2_out/2.d0/msb22
      dthetab_out = dthetab_out/2.d0/dcos(2*thetab_in)

      end

C-}}}

C-{{{ function dAbosfinmb

      function dAbosfinmbHM(deltamb,mu_in)
      implicit none
      double precision dAbosfinmbHM,tanb,b0fin,deltamb,mu_in

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      tanb = dtan(beta)

      dAbosfinmbHM = - ((4*lb1*mb*msb12 + (3*deltamb+ 8*mb)
     - *(msb12 - msb22) - 4*lb2*mb*msb22)*(Ab + muSUSY/tanb) + 
     - 2*mb*(Ab*(mb2 + mgl**2 - msb12) + mgl*(-msb12 + msb22) + 
     - ((mb2 + mgl**2 - msb12)*muSUSY)/tanb)*b0fin(msb12,mb,mgl,mu_in)
     - - 2*mb*(Ab*(mb2 + mgl**2 - msb22) + mgl*(msb12 - msb22) + 
     - ((mb2 + mgl**2 - msb22)*muSUSY)/tanb)*b0fin(msb22,mb,mgl,mu_in))/
     - (3.d0*mb*(msb12 - msb22))
      end

      function dAbosfinmbDS(deltamb,mu_in)
      implicit none
      double precision dAbosfinmbDS,tanb,deltamb,mu_in

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      tanb = dtan(beta)

      dAbosfinmbDS = (2*mgl*(4
     -     + ((mgl**2 - msb12)*dlog(dabs(1 - msb12/mgl**2)))/msb12
     -     + ((mgl**2 - msb22)*dlog(dabs(1 - msb22/mgl**2)))/msb22
     -     - (3*(deltamb-(dmbsbos(1)+dmbsbos(2)))
     -     /mb*(Ab+muSUSY/dtan(beta)))/(2.d0*mgl)
     -     + 4*dlog(mu_in/mgl) ))/3.d0
      end

C-}}}
C-{{{ function dAbosfintb

      function dAbosfintbHM(deltatb,mu_in)
      implicit none
      double precision dAbosfintbHM,tanb,b0fin,dmsb12,dmsb22,deltatb,
     &mu_in

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      tanb = dtan(beta)
      dmsb12 = 2*dmsb1(1)*msb12
      dmsb22 = 2*dmsb2(1)*msb22

      dAbosfintbHM = ((6*c2thetab*deltatb*(msb12 - msb22) + 
     -   (3*dmsb12 - 3*dmsb22 + 8*msb12 + 4*lb1*msb12 - 8*msb22 - 
     -   4*lb2*msb22)*s2thetab)*(Ab + muSUSY/tanb) - 
     -   (2*s2thetab*b0fin(msb22,mb,mgl,mu_in)*
     -   ((mb2 + mgl**2 - msb22)*muSUSY*dcos(beta) + 
     -   (Ab*(mb2 + mgl**2 - msb22) + mgl*(msb12 - msb22))*dsin(beta)))/
     -   dsin(beta) + (2*s2thetab*b0fin(msb12,mb,mgl,mu_in)*
     -   ((mb2 + mgl**2 - msb12)*muSUSY*dcos(beta) + 
     -   (Ab*(mb2 + mgl**2 - msb12) + mgl*(-msb12 + msb22))*dsin(beta)))
     -   /dsin(beta))/
     -   (6*Ab*mb - 3*(msb12 - msb22)*s2thetab + (6*mb*muSUSY)/tanb) 
      end

      function dAbosfintbDS(deltatb,mu_in)
      implicit none
      double precision dAbosfintbDS,deltatb,mu_in

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      dAbosfintbDS = (2*mgl*(4
     -  + ((mgl**2 - msb12)*dlog(dabs(1 - msb12/mgl**2)))/msb12
     -  + ((mgl**2 - msb22)*dlog(dabs(1 - msb22/mgl**2)))/msb22
     -  - (3*(deltatb - dthetabos(1))/dtan(2*thetab)*
     -    (Ab+muSUSY/dtan(beta)))/mgl
     -  + 4*dlog(mu_in/mgl) ))/3.d0
      end

C-}}}
C-{{{ function mbMSDRtrans

      function mbMSDRtrans(mb_in,orderin)
      implicit none
      double precision mbMSDRtrans, mb_in, orderin

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      mbMSDRtrans = mb_in * (1.d0 - 1.d0/3.d0*alphamuR/Pi)

      if (orderin.eq.1) then
      mbMSDRtrans = mbMSDRtrans - mb_in*(29.d0/72.d0 *(alphamuR/Pi)**2)
      end if

      end

C-}}}
C-{{{ function dmbfin

      function dmbfin(smflag,drflag,mb_in,mu)
      implicit none
      double precision dmbfin,b0fin,mb_in,mb2_in
      double precision lb_in, lb1_in, lb2_in, mu
      double precision lt_in, lt1_in, lt2_in, lg_in
      logical smflag,drflag

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      mb2_in = mb_in**2
      lb_in = dlog(mu**2/mb2_in)
      lb1_in = dlog(mu**2/msb12)
      lb2_in = dlog(mu**2/msb22)
      lt_in = dlog(mu**2/mt2)
      lt1_in = dlog(mu**2/mst12)
      lt2_in = dlog(mu**2/mst22)
      lg_in = 2*dlog(mu/mgl)

      dmbfin = (- 2.d0*(1.d0 +lg_in)*mgl**2+
     -     msb12 + lb1_in*msb12 + msb22 + lb2_in*msb22 +
     -     (mb2_in + mgl**2 - msb12 - 2.d0*mb_in*mgl*s2thetab)*
     -     b0fin(mb2_in,mgl,dsqrt(msb12),mu) 
     -    + (mb2_in + mgl**2 - msb22 +
     -     2*mb_in*mgl*s2thetab)*b0fin(mb2_in,mgl,dsqrt(msb22),mu))
     -    /(6.d0*mb_in)

      if (smflag.eqv.(.true.)) then
      if (drflag.eqv.(.true.)) then
         dmbfin = dmbfin - 2.d0*(5.d0+ 3.d0*lb_in)*mb2_in/(6.d0*mb_in)
      else
         dmbfin = dmbfin - 2.d0*(4.d0+ 3.d0*lb_in)*mb2_in/(6.d0*mb_in)
      end if
      end if
      end

C-}}}
C-{{{ function dmbnontanb - non-tanbeta enhanced terms only

      function dmbnontanb(smflag,drflag,mb_in,mu)
      implicit none
      double precision dmbnontanb,b0fin,mb_in,mb2_in
      double precision lb_in, lb1_in, lb2_in, mu
      double precision lt_in, lt1_in, lt2_in, lg_in
      logical smflag,drflag

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      mb2_in = mb_in**2
      lb_in = dlog(mu**2/mb2_in)
      lb1_in = dlog(mu**2/msb12)
      lb2_in = dlog(mu**2/msb22)
      lt_in = dlog(mu**2/mt2)
      lt1_in = dlog(mu**2/mst12)
      lt2_in = dlog(mu**2/mst22)
      lg_in = 2*dlog(mu/mgl)

      dmbnontanb = (- 2.d0*(1.d0+lg_in)*mgl**2+
     -     msb12 + lb1_in*msb12 + msb22 + lb2_in*msb22 +
     -     (mb2_in + mgl**2 - msb12 - 4.d0*mb2_in*mgl*Ab/(msb12-msb22))*
     -     b0fin(mb2_in,mgl,dsqrt(msb12),mu) 
     -      + (mb2_in + mgl**2 - msb22 + 4*mb2_in*mgl*Ab/(msb12-msb22))
     -     *b0fin(mb2_in,mgl,dsqrt(msb22),mu))
     -    /(6.d0*mb_in)

      if (smflag.eqv.(.true.)) then
      if (drflag.eqv.(.true.)) then
         dmbnontanb = dmbnontanb - 2.d0*(5.d0+ 3.d0*lb_in)*mb2/(6.d0*mb)
      else
         dmbnontanb = dmbnontanb - 2.d0*(4.d0+ 3.d0*lb_in)*mb2/(6.d0*mb)
      end if
      end if

      end

C-}}}
C-{{{ function dmbtanb - tanbeta enhanced terms only

      function dmbtanb(mb_in,mu)
      implicit none
      double precision dmbtanb,b0fin,mb_in,mb2_in,mu

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      mb2_in = mb_in**2

      dmbtanb = (4*mb2_in*mgl*muSUSY*tan(beta)/(msb12-msb22)
     -     *b0fin(mb2_in,mgl,dsqrt(msb12),mu)  -
     -     4*mb2_in*mgl*muSUSY*tan(beta)/(msb12-msb22)
     -     *b0fin(mb2_in,mgl,dsqrt(msb22),mu))
     -    /(6.d0*mb_in)
      end

C-}}}
C-{{{ function dthetabosfin

      function dthetabosfin()
      implicit none
      double precision dthetabosfin,b0fin

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      dthetabosfin = (c2thetab*((-((1 + lb1)*msb12) + 
     -   (1 + lb2)*msb22)*s2thetab + 
     -   2*mb*mgl*(b0fin(msb12,mb,mgl,muR) + b0fin(msb22,mb,mgl,muR))))/
     -   (3.d0*(msb12 - msb22))
      end

C-}}}
C-{{{ function dmsb1osfin

      function dmsb1osfin()
      implicit none
      double precision dmsb1osfin,b0fin

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      dmsb1osfin = -((1 + lb)*mb2 + (1 + lg)*mgl**2 + (3 + lb1)*msb12 + 
     -     2*cthetab**2*((1 + lb1)*msb12 - (1 + lb2)*msb22)*sthetab**2 + 
     -(mb2 + mgl**2 - msb12 - 2*mb*mgl*s2thetab)*b0fin(msb12,mb,mgl,muR)
     -      )/(3.d0*dsqrt(msb12))
      end

C-}}}
C-{{{ function dmsb2osfin

      function dmsb2osfin()
      implicit none
      double precision dmsb2osfin,b0fin

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      dmsb2osfin = -((1 + lb)*mb2 + (1 + lg)*mgl**2 + (3 + lb2)*msb22 + 
     -  2*cthetab**2*(-((1 + lb1)*msb12) + (1 + lb2)*msb22)*sthetab**2 + 
     -(mb2 + mgl**2 - msb22 + 2*mb*mgl*s2thetab)*b0fin(msb22,mb,mgl,muR)
     -     )/(3.d0*dsqrt(msb22))
      end

C-}}}

C-{{{ dAtosfins2

      function dAtosfins2()
c     finite part of delta A_b^OS * sin(2 thetat)
      implicit none
      double precision dAtosfins2,dmst12osfin,dmst22osfin,dthetatosfin,
     &dmtosfin

      include 'commons/common-quark.f'
      include 'commons/common-ren.f'
      include 'commons/common-inputoutput.f'

      dAtosfins2 = (s2thetat*(dmst12osfin()-dmst22osfin())/(mst12-mst22)
     & + 2*c2thetat*dthetatosfin()-dmtosfin(.true.)*s2thetat/dsqrt(mt2)
     & ) * (At - muSUSY/dtan(beta))
      end

C-}}}
C-{{{ dmtosfin

      function dmtosfin(smflag)
      implicit none
      double precision dmtosfin,b0fin
      logical smflag

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      dmtosfin = (- 2*(1 + lg)*mgl**2 +
     -     mst12 + lt1*mst12 + mst22 + lt2*mst22 +
     -     (mt2 + mgl**2 - mst12 - 2*mt*mgl*s2thetat)*
     -     b0fin(mt2,mgl,dsqrt(mst12),muR) + (mt2 + mgl**2 - mst22 +
     -     2*mt*mgl*s2thetat)*b0fin(mt2,mgl,dsqrt(mst22),muR))
     -    /(6.d0*mt)
      if (smflag.eqv.(.true.)) then
         dmtosfin = dmtosfin - 2*(5 + 3*lt)*mt2/(6.d0*mt)
      endif
      end

C-}}}
C-{{{ dthetatosfin

      function dthetatosfin()
      implicit none
      double precision dthetatosfin,b0fin

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      dthetatosfin = (c2thetat*((-((1 + lt1)*mst12) +
     -   (1 + lt2)*mst22)*s2thetat + 
     -   2*mt*mgl*(b0fin(mst12,mt,mgl,muR) + b0fin(mst22,mt,mgl,muR))))/
     -   (3.d0*(mst12 - mst22))
      end

C-}}}
C-{{{ dmst12osfin

      function dmst12osfin()
      implicit none
      double precision dmst12osfin,b0fin

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      dmst12osfin = -((1 + lt)*mt2 + (1 + lg)*mgl**2 + (3 + lt1)*mst12 + 
     -     2*cthetat**2*((1 + lt1)*mst12 - (1 + lt2)*mst22)*sthetat**2 + 
     -(mt2 + mgl**2 - mst12 - 2*mt*mgl*s2thetat)*b0fin(mst12,mt,mgl,muR)
     -      )*2/3.d0
      end

C-}}}
C-{{{ dmst22osfin

      function dmst22osfin()
      implicit none
      double precision dmst22osfin,b0fin

      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-ren.f'
      include 'commons/common-consts.f'

      dmst22osfin = -((1 + lt)*mt2 + (1 + lg)*mgl**2 + (3 + lt2)*mst22 + 
     -  2*cthetat**2*(-((1 + lt1)*mst12) + (1 + lt2)*mst22)*sthetat**2 + 
     -(mt2 + mgl**2 - mst22 + 2*mt*mgl*s2thetat)*b0fin(mst22,mt,mgl,muR)
     -     )*2/3.d0
      end

C-}}}

C-{{{ alphas

      real*8 function alphaS(rmurl)
      implicit none
      double precision rmurl,api0,apiout

      include 'commons/common-consts.f'      
      include 'commons/common-vars.f' 

      api0 = AlfasMZ/Pi

      call runalpha(api0,mz,rmurl,nf,4,0,apiout)

      alphaS = apiout*Pi

      end

C-}}}
C-{{{ subroutine polemass:

      subroutine polemasshm(mqmq,apimq,nloop,mqpole)
c..
c..   Computes the pole mass mqpole from the MS-bar mass mqmq = mq(mq).
c..   apimq = alpha_s(mqmq)/pi.
c..
c      implicit real*8 (a-z)
      implicit none
      integer nloop
      double precision mqmq,apimq,mqpole,lmum

      include 'commons/common-consts.f'
      include 'commons/common-vars.f'

      lmum = 0.d0

      if (nloop.eq.0) then
         mqpole = mqmq
      elseif (nloop.eq.1) then
         mqpole = mqmq*( 1 + apimq*cf*(1 + (3*lmum)/4.d0) )
      elseif (nloop.eq.2) then
         mqpole = mqmq*( 1 + apimq*cf*(1 + (3*lmum)/4.d0)
     &        + apimq**2*(
     &        + cf**2*( -71/128.d0 - (9*lmum)/32.d0 + (9*lmum**2)/32.d0
     &        +(15*z2)/8.d0  - 3*ln2*z2 + (3*z3)/4.d0 )+ cf*ca*( 1111
     &        /384.d0 + (185*lmum)/96.d0 + (11*lmum**2)/32.d0  - z2/2.d0
     &        + (3*ln2*z2)/2.d0 - (3*z3)/8.d0 )+ cf*tr*( -3/4.d0 - (71
     &        *nf)/96.d0 - (13*lmum*nf)/24.d0 -(lmum**2*nf)/8.d0 + (3*z2
     &        )/2.d0 - (nf*z2)/2.d0) ) )
      elseif (nloop.eq.3) then
         mqpole = mqmq*( 1 + apimq*cf*(1 + (3*lmum)/4.d0)
     &        + apimq**2*(
     &        + cf**2*( -71/128.d0 - (9*lmum)/32.d0 + (9*lmum**2)/32.d0
     &        +(15*z2)/8.d0  - 3*ln2*z2 + (3*z3)/4.d0 )+ cf*ca*( 1111
     &        /384.d0 + (185*lmum)/96.d0 + (11*lmum**2)/32.d0  - z2/2.d0
     &        + (3*ln2*z2)/2.d0 - (3*z3)/8.d0 )+ cf*tr*( -3/4.d0 - (71
     &        *nf)/96.d0 - (13*lmum*nf)/24.d0 -(lmum**2*nf)/8.d0 + (3*z2
     &        )/2.d0 - (nf*z2)/2.d0) )
     &        + apimq**3*(.6527d0*(nf-1.d0)**2 - 26.655d0*(nf-1.d0)
     &        +190.595d0) ) 
      else
         write(6,*) '<function polemass>: nloop = ',nloop
     &        ,' not implemented'
      endif

      end

C-}}}
C-{{{ subroutine getmass

      subroutine getmass()
c      from ggh@nnlo
      implicit none
      double precision apim, apimt, apimuB, apimuD
      double precision alphaS

      include 'commons/common-consts.f'
      include 'commons/common-quark.f'
      include 'commons/common-vars.f'
      include 'commons/common-int.f'
      include 'commons/common-ren.f'

c      external runmass
c      apimuR = alphamuR/Pi
      apimuR = alphaS(muR)/Pi

      apim  = alphaS(mcmc)/Pi
      call polemasshm(mcmc,apim,3,mcos) !3-loop conversion

!mass mbos
      apim  = alphaS(mbmb)/Pi
!      call polemasshm(mbmb,apim,1,mbos) !1-loop conversion
!      call polemasshm(mbmb,apim,2,mbos) !2-loop conversion
      call polemasshm(mbmb,apim,3,mbos) !3-loop conversion


!mass mbmuR
!      call runmass(mbmb,apim,apimuR,nf,norderggh+1,mbMSbarmuR)
      call runmass(mbmb,apim,apimuR,nf,4,mbMSbarmuR) !4-loop running

!mass mbmuB
      apimuB  = alphaS(muB)/Pi
      call runmass(mbmb,apim,apimuB,nf,4,mbMSbarmuB) !4-loop running

!mass mbmt
      apimt  = alphaS(mt)/Pi
      call runmass(mbmb,apim,apimt,nf,4,mbMSbarmt) !4-loop running

      mc2 = mcos**2

      end

C-}}}
