! This file is main file of SusHi.
! 
! written by Robert Harlander, Stefan Liebler, Hendrik Mantler
! Bergische Universitaet Wuppertal
! Version 1.0.0
! December 2012
! 
      program SUSHI

      implicit none
      integer ndm,ncall,itmx,nprn,ncall1,itmx1,nprn1,nmfv,count1
      logical ppcoll,fhflagout
      double precision ms42,cthetab4,sthetab4,cthetat4,sthetat4
      double precision msbp1,msbp2,mstp1,mstp2
      double precision xl(10),xu(10),acc,lmurmt
      double precision alphas
      double precision SigLO,errLO,Sig_bb,err_bb,Sig_bg,err_bg
      double precision sigmaggh,Sig_total,err_total
      double precision delmb,delAb,ifunc
      double precision wAex(2),wLHex(2),wHHex(2)

      !FH specific variables
      double precision MASf(6,4),MCha(2),MNeu(4),MHtree(4)
      double precision MHiggs(4),SAtree,MSb(2),MbSL2
      double complex UASf(6,6,4),UCha(2,2),VCha(2,2),ZNeu(4,4)
      double complex SAeff,UHiggs(3,3),ZHiggs(3,3),USb(2,2)
      double complex DeltaMB,Deltab

      !common variables used by various routines
      include 'commons/common-inputoutput.f'
      include 'commons/common-consts.f'
      include 'commons/common-vars.f'
      include 'commons/common-quark.f'
      include 'commons/common-int.f'
      include 'commons/common-ren.f'
      include 'commons/common-readdata.f'

      common /coll/ ppcoll
      common /bveg1/ xl,xu,acc,ndm,ncall,itmx,nprn

      pi = 3.14159265358979323846264338328d0
      z2 = 1.6449340668482264364724d0
      z3 = 1.2020569031595942853997d0
      z4 = 1.0823232337111381915160d0
      z5 = 1.0369277551433699263314d0
      ca = 3.d0
      cf = 4/3.d0
      tr = 1/2.d0

      call printcopysushi(1,6)
      !read-in of parameters using readinsushi from inputoutput.f
      call get_command_argument(1,jfilein)
      call get_command_argument(2,jfnameact)

      if ((len_trim(jfilein).eq.0).or.(len_trim(jfnameact).eq.0)) then
       write(*,*) "SusHi must be called with two arguments:"
       write(*,*) "./SusHi in.in out.out"
       write(*,*) "The input-file in.in has to be provided"
       write(*,*) "in the SLHA-standard. Example files: inFH.in, ..."
       stop
      end if

      call readinsushi(fhflagout)

      !---MSSM
      if ((model.eq.1).or.((model.eq.0)
     &.and.(fhflagout.eqv.(.true.)))) then
      !FH specific routines
#ifdef NOFEYNHIGGS
         if (fhflagout.eqv.(.true.)) then
            write(*,*) "You compiled SusHi without FeynHiggs,"
            write(*,*) "but tried to call a FeynHiggs specific",
     &" input-file."
            write(*,*) "Please use another input-file. SusHi stops!"
            stop
         end if
#else
         if (fhflagout.eqv.(.true.)) then
            call FHSetFlags(error,
     &           mssmpart, fieldren, tanbren, higgsmix, p2approx,
     &           looplevel, runningMT, botResum, tlCplxApprox)

            if(error.ne.0) then
               write(*,105)
               write(*,*) 'Error in FHSetFlags ', error
               write(*,105)
               stop
            endif

            call FHSetSMPara(error,
     &           invAlfa, AlfasMZ, GF,
     &           ME, MU, MD, MM, MC, MS, ML, mbmb,
     &           MW, MZ,
     &           CKMlambda, CKMA, CKMrhobar, CKMetabar)

            if(error.ne.0) then
               write(*,105)
               write(*,*) 'Error in FHSetSMPara ', error
               write(*,105)
               stop
            endif

            call FHSetPara(error,scalefactor,MT,tanb,MA0,MHp,
     &           M3SL,M3SE,M3SQ,M3SU,M3SD,
     &           M2SL,M2SE,M2SQ,M2SU,M2SD,
     &           M1SL,M1SE,M1SQ,M1SU,M1SD,
     &           MUE,cAtau,cAt,cAb,
     &           cAmu,cAc,cAs,cAe,cAu,cAd,M_1,M_2,M_3,Qtau,Qt,Qb)

            if(error.ne.0) then
               write(*,105)
               write(*,*) 'Error in FHSetPara ', error
               write(*,105)
               stop
            endif

            call FHGetPara(error, nmfv, MASf, UASf,
     &           MCha, UCha, VCha, MNeu, ZNeu, DeltaMB, MGl,
     &           MHtree, SAtree)

            if(error.ne.0) then
               write(*,105)
               write(*,*) 'Error in FHGetPara ', error
               write(*,105)
               stop
            endif

            call FHHiggsCorr(error,MHiggs,SAeff,UHiggs,ZHiggs)

            if(error.ne.0) then
               write(*,105)
               write(*,*) 'Error in FHHiggsCorr ', error
               write(*,105)
               stop
            endif

            call FHGetTLPara(error, MSb, USb, MbSL2, Deltab)
      
            if(error.ne.0) then
               write(*,105)
               write(*,*) 'Error in FHGetTLPara ', error
               write(*,105)
               stop
            endif

            dMLb = MbSL2-M3SQ**2

            !Set delmb equal to DeltaMB from FH - own calculation in renscheme.f
            delmb = DeltaMB

            Msbot1 = Msb(1)
            Msbot2 = Msb(2)

            cthetab = Real(Usb(1,1))
            sthetab = Real(Usb(1,2))

            alpha = dasin(Real(SAeff))
            Mstop1 = MASf(3,3)
            Mstop2 = MASf(6,3)
            cthetat = Real(UASf(3,3,3))
            sthetat = Real(UASf(3,6,3))
            muSUSY = Real(MUE)

      !get thetat from cthetat and sthetat, 0<=thetat<Pi
            if (cthetat.gt.0.d0) then
               thetat = dasin(sthetat)
            else
               thetat = Pi-dasin(sthetat)
            endif

      !thetab from OS-sbottom masses in renormalize()

            if (pseudo.eq.0) then
               Mh = MHiggs(1)   !h
            else if (pseudo.eq.1) then
               Mh = MHiggs(3)   !A
            else if (pseudo.eq.2) then
               Mh = MHiggs(2)   !H

c               alpha = alpha-Pi/2.d0
c               pseudo = 0

            end if
         end if

#endif
      !end of FH specific input
      end if

      if ((Mh.Le.(90.d0)).Or.(Mh.Ge.(1000.d0))) then
         write(*,*) "Warning: You called SusHi with a small Higgs mass."
         write(*,*) "Results might not be reasonable."
         write(*,*) "Electroweak contributions not implemented."
c      stop
      end if

      write(*,105)
      write(*,106) 'Higgs mass:',Mh
      write(*,105)

      q0 = q0c + q01*Mstop1 + q02*Mstop2

      !determine whether to use the heavy top limit 
      !m_t -> inf in the top-cont.
      htl = .false.

      !call ffini

      !settings in case of a fourth generation
      if((yukfac(4).ne.0.d0).or.(yukfac(5).ne.0.d0)) then

         if(mbp.ne.mtp) then
            write(*,105)
            write(*,*) 'Warning: masses of the 4th generation quarks '
            write(*,*) 'are not equal. Setting m_b` = m_t`. '
            write(*,105)
            mbp = mtp
         endif

         ms42 = MSUSY**2+mtp**2
         mstp12 = ms42 - MSUSY**2
         mstp22 = ms42 + MSUSY**2
         msbp12 = mstp12
         msbp22 = mstp22
         mstp1 = dsqrt(mstp12)
         mstp2 = dsqrt(mstp22)
         msbp1 = dsqrt(msbp12)
         msbp2 = dsqrt(msbp22)

         Mh = dsqrt(
     &          Mh**2
     &        + (3/2.d0 * dlog(ms42/Mbp**2) * Mbp**4
     &        +  3/2.d0 * dlog(ms42/Mtp**2) * Mtp**4 ) 
     &        / (Pi/dsqrt(dsqrt(2.d0)*GF))**2 )

         if(Mh.gt.350.d0) then
            write(*,105)
            write(*,*) "Error: Higgs mass too large.", Mh
            write(*,105)
            stop
         endif
      endif

      mt2 = MT**2

      mst12 = Mstop1**2
      mst22 = Mstop2**2     
      mcmc = Mcharm
      mbp2 = Mbp**2
      mtp2 = Mtp**2
      msb12 = Msbot1**2
      msb22 = Msbot2**2     

      !definition of renormalization scale and variables
      muR = Mh * muRfactor
      muF = Mh * muFfactor
      
      if (mbrunyuk.eq.1) then
         muB = mbmb
      else if (mbrunyuk.eq.2) then
         muB = muR
      else
         muB = Mh
      end if

      mh2 = Mh**2
      z = mh2/cme**2
      sigmanull = GF/288/dsqrt(2.d0)/Pi
     &     * 0.38937966d9 ! last factor: conversion gev -> pb
      nf = 5.d0
      betanull = 11/12.d0*ca-nf/6.d0
      ln2 = dlog(2.d0)
      lfh = dlog(muF**2/mh2)
      lfr = dlog(muF**2/muR**2)

      !settings for VEGAS and PDF initialization
      ncall1 = ncall
      itmx1 = itmx
      nprn1 = nprn

C-{{{ cuts

      !definition of cuts
      subtr = .true.

      minpt = 0.d0
      maxpt = cme*(1-z)/2.d0
      minrap = 0.d0
      maxrap = -dlog(z)/2.d0

      if(dist.ge.2) then
         if((maxrapc.lt.0.d0).or.(
     &     (maxrapc.ge.maxrap).and.(.not.pseudorap) )) then
            write(*,105)
            write(*,*) 'Error: y out of range. '
            write(*,*) 'Abs[y] < ', maxrap
            write(*,105)
            stop
         endif         
      endif

      if(dist.eq.1) then
         subtr = .false.
         if((maxptc.lt.0.d0).or.(maxptc.ge.maxpt)) then
            write(*,105)
            write(*,*) 'Error: pt out of range. '
!            write(*,*) '30 GeV < pt < ', maxpt, ' GeV '
            write(*,105)
            stop
         endif
      elseif(dist.eq.3) then
         subtr = .false.
         if(maxptc.gt.
     &dsqrt(mh2*(dexp(2*maxrapc)*(1/z+z)-1-dexp(4*maxrapc)))/
     &(1+dexp(2*maxrapc))) then
            write(*,105)
            write(*,*) 'Error: pt out of range. '
            write(*,*) 'for y = ', maxrapc, ', pt must satisfy'
            write(*,*) '0 < pt < ', dsqrt(mh2*(dexp(2*maxrapc)
     &*(1/z+z)-1-dexp(4*maxrapc)))/(1+dexp(2*maxrapc))
            write(*,105)
            stop
         endif         
      endif

      if(ptc.eq.0) then
      !no pt-cut
         ptcut = .false.
         mincut = .false.
      elseif(ptc.eq.1) then
      !pt > ptmin
         if((minptc.lt.0.d0).or.(minptc.ge.maxpt)) then
            write(*,105)
            write(*,*) 'Error: ptmin out of range. '
            write(*,105)
            stop
         endif
         ptcut = .true.
         mincut = .true.
         subtr = .false.
         minpt = minptc
      elseif(ptc.eq.2) then
      !pt < ptmax
         if(maxptc.lt.0.d0) then
            write(*,105)
            write(*,*) 'Error: ptmax out of range. '
            write(*,105)
            stop
         endif
         ptcut = .true.
         mincut = .false.
         minpt = 0.d0
         if(maxptc.gt.maxpt) then
            write(*,105)
            write(*,*) 'Warning: ptmax too large. '
            write(*,*) 'maximal value for ptmax is ', maxpt, ' GeV. '
            write(*,105)
         else
            maxpt = maxptc
         endif
      elseif(ptc.eq.3) then
      !ptmin < pt < ptmax
         if((minptc.lt.0.d0).or.(minptc.ge.maxpt)) then
            write(*,105)
            write(*,*) 'Error: ptmin out of range. '
            write(*,105)
            stop
         endif
         if(maxptc.lt.0.d0) then
            write(*,105)
            write(*,*) 'Error: ptmax out of range. '
            write(*,105)
            stop
         endif
         ptcut = .true.
         mincut = .true.
         subtr = .false.
         minpt = minptc
         if(maxptc.gt.maxpt) then
            write(*,105)
            write(*,*) 'Warning: ptmax too large. '
            write(*,*) 'maximal value for ptmax is ', maxpt, ' GeV. '
            write(*,105)
         else
            maxpt = maxptc
         endif
         if(maxpt.le.minpt) then
            write(*,105) 
            write(*,*) 'Error: ptmin larger then ptmax. '
            write(*,105) 
            stop
         endif
      else
         write(*,105)
         write(*,*) 'Error: bad value in Block DISTRIB. '
         write(*,105)
         stop
      endif
      
      if(rapc.eq.0) then
      !no ycut
         rapcut = .false.
      elseif(rapc.eq.1) then
      !Abs[y] < ymax
         if(maxrapc.lt.0.d0) then
            write(*,105)
            write(*,*) 'Error: ymax out of range. '
            write(*,105)
            stop
         endif            
         rapcut = .true.
         minrap = 0.d0
         if((maxrapc.gt.maxrap).and.(.not.pseudorap)) then
            write(*,105)
            write(*,*) 'Warning: ymax too large. '
            write(*,*) 'maximal value for ymax is ', maxrap, '. '
            write(*,105)
         else
            maxrap = maxrapc
         endif
      elseif(rapc.eq.2) then
      !Abs[y] > ymin
         if(pseudorap) then
            if(minrapc.lt.0.d0) then
               write(*,105)
               write(*,*) 'Error: ymin out of range. '
               write(*,*) '0 < ymin'
               write(*,105)
               stop
            endif
            maxrap = 1.d100
         else
            if((minrapc.lt.0.d0).or.(minrapc.ge.maxrap)) then
               write(*,105)
               write(*,*) 'Error: ymin out of range. '
               write(*,*) '0 < ymin < ', maxrap
               write(*,105)
               stop
            endif
         endif
         rapcut = .true.
         minrap = minrapc
      elseif(rapc.eq.3) then
      !subtr = .false.
      !ymin < Abs[y] < ymax
         if(pseudorap) then
            if(minrapc.lt.0.d0) then
               write(*,105)
               write(*,*) 'Error: ymin out of range. '
               write(*,*) '0 < ymin'
               write(*,105)
               stop
            endif
            maxrap = maxrapc
         else
            if((minrapc.lt.0.d0).or.(minrapc.ge.maxrap)) then
               write(*,105)
               write(*,*) 'Error: ymin out of range. '
               write(*,*) '0 < ymin < ', maxrap
               write(*,105)
               stop
            endif
            if(maxrapc.gt.maxrap) then
               write(*,105)
               write(*,*) 'Warning: ymax too large. '
               write(*,*) 'maximal value for ymax is ', maxrap, '. '
               write(*,105)
            else
               maxrap = maxrapc
            endif
         endif
         if(maxrapc.lt.0.d0) then
            write(*,105)
            write(*,*) 'Error: ymax out of range. '
            write(*,105)
            stop
         endif            
         rapcut = .true.
         minrap = minrapc
      else
         write(*,105)
         write(*,*) 'Error: bad value in Block DISTRIB. '
         write(*,105)
         stop         
      endif

      pty = .false.
      juser = .false.

      if(rapcut.and.(.not.pseudorap)) then
         pty = .true.
      endif

      if(juser) then
         pty = .false.
      endif

      ptsave = maxptc
      ysave = maxrapc

c      pty = .false.

C-}}}
C-{{{ calculation of cross sections - calling ggh@nnlo and bbh@nnlo
      do count1=1,3
         SUbbh(count1) = 0.D0
         SUbbhsusy(count1) = 0.D0
         SUbbherr(count1) = 0.D0
         SUbbhsusyerr(count1) = 0.D0
         SUggh(count1) = 0.D0
         SUggherr(count1) = 0.D0
      end do

c$$$      if ((dist.eq.0).and.(ptc.eq.0).and.(rapc.eq.0)) then
c$$$
c$$$         write(*,105)
c$$$         write(*,*) "Calling ggh@nnlo and bbh@nnlo"
c$$$         write(*,105)
c$$$
c$$$      !calling at LO
c$$$         call XSgghbbh(0,SUggh(1),SUggherr(1),SUbbh(1))
c$$$
c$$$      !calling at NNLO
c$$$         call XSgghbbh(2,SUggh(3),SUggherr(3),SUbbh(3))
c$$$
c$$$      !calling at NLO - PDFs also for subsequent NLO calculations
c$$$         call XSgghbbh(1,SUggh(2),SUggherr(2),SUbbh(2))
c$$$      end if

      !write(*,*) "SUggh",SUggh
      !write(*,*) "SUbbh",SUbbh

      itmx = itmx1
      nprn = nprn1

      !Differential distributions for bbh
      If ((dist.eq.0).and.((ptc.gt.0).or.(rapc.gt.0))) then
      If (norderbbh.ge.0) then

       write(*,105)
       write(*,*) "Performing (N)LO bbh calculation with cuts"
       write(*,105)

       if (ptc.gt.0) then
       write(*,*) "Applied p_t cuts:           ",minpt,maxpt
       end if
       if (rapc.gt.0) then
       write(*,*) "Applied pseudorapidity cuts:",minrap,maxrap
       end if

       call SUinitpdf(SUpdfs(1),SUiset)
       call runCouplings(muR,0)
       call bbhdiffinit(0,cme,mh,0.d0,mt
     & ,-1,-1,0,pseudorap,minpt,maxpt,minrap,maxrap,muR,muF)
       call Sigma_bbHNLO(ncall1,SigLO,errLO,Sig_bb,err_bb,Sig_bg,err_bg
     & ,SUbbh(1),SUbbherr(1))

       if (norderbbh.ge.1) then
       call SUinitpdf(SUpdfs(2),SUiset)
       call runCouplings(muR,1)
       call bbhdiffinit(1,cme,mh,0.d0,mt
     & ,-1,-1,0,pseudorap,minpt,maxpt,minrap,maxrap,muR,muF)
       call Sigma_bbHNLO(ncall1,SigLO,errLO,Sig_bb,err_bb,Sig_bg,err_bg
     & ,SUbbh(2),SUbbherr(2))
       end if

       if (norderbbh.eq.2) then
       write(*,105)
       write(*,*) "Warning: Differential cross sections for bb->H are"
       write(*,*) "not available at NNLO. NLO results are given."
       write(*,105)
       end if

      end if
      end if

      ncall = ncall1

C-}}}
C-{{{ settings for NLO calculation

      if (norderggh.eq.0) then
c         write(*,105)
c         write(*,*) "Performing LO ggh calculation"
c         write(*,105)
         call SUinitpdf(SUpdfs(1),SUiset)
         call checkorder(0)
      else if (norderggh.gt.0) then
c         write(*,105)
c         write(*,*) "Performing NLO ggh calculation"
c         write(*,105)
         call SUinitpdf(SUpdfs(2),SUiset)
         call checkorder(1)
      endif
      call SUSHI_BERNINI(18)

      !settings for a fourth generation
      if((yukfac(4).ne.0.d0).or.(yukfac(5).ne.0.d0)) then

         cthetat4 = 0.d0
         sthetat4 = 1.d0
         cthetab4 = 0.d0
         sthetab4 = 1.d0

         call evalcsusy(Mbp,mstp1,msbp2,Mtp,Mstp1,Mstp2,Mgl,beta,
     &Mw,Mz,alpha,Mh,muR,muSUSY,cthetat4,sthetat4,cthetab4,sthetab4,
     &c1sm14,c1sm24,c1sm34,c1susy1t4,c1susy2t4,c1susy1b4,c1susy2b4,
     &gb4,gt4,gb14,gbh124,gbh214,gb24,gt14,gth124,gth214,gt24,pseudo)

         gbpe  = (c1susy1b4-gb4*c1sm14) * yukfac(9)
         gbpe2 = (c1susy2b4-gb4*c1sm24) * yukfac(9)

         gtpe  = (c1susy1t4-gt4*c1sm14) * yukfac(8)
         gtpe2 = (c1susy2t4-gt4*c1sm24) * yukfac(8)

         gbp = gb4 * yukfac(5)
         gtp = gt4 * yukfac(4)
         gbp1 = gb14 * yukfac(9)
         gbp2 = gb24 * yukfac(9)
         gtp1 = gt14 * yukfac(8)
         gtp2 = gt24 * yukfac(8)

      else

         gbp = 0.d0
         gtp = 0.d0
         gbp1 = 0.d0
         gbp2 = 0.d0
         gtp1 = 0.d0
         gtp2 = 0.d0
         gbpe = 0.d0
         gbpe2 = 0.d0
         gtpe = 0.d0
         gtpe2 = 0.d0

      endif

      call getmass()

      if (mbossave.gt.0.D0) then
       mbos = mbossave
      end if

      c1eff0rd = 1.d0
      c1eff1rd = 2.75d0

      !only for MSSM
      if (model.ne.0) then

      !Ab either taken from input file or FH
         Ab = Real(cAb)
         At = Real(cAt)
         beta = datan(tanb)

         call renormalize(fhflagout,delmb,delAb)

         q0 = q0c + q01*Mstop1 + q02*Mstop2

         call evalcsusy(mbsb,msbot1,msbot2,mt,mstop1,mstop2,Mgl,beta,
     &Mw,Mz,alpha,Mh,muR,muSUSY,cthetat,sthetat,cthetab,sthetab,
     &c1sm13,c1sm23,c1sm33,c1susy1t3,c1susy2t3,c1susy1b3,c1susy2b3,
     &gb3,gt3,gb13,gbh123,gbh213,gb23,gt13,gth123,gth213,gt23,pseudo)

         gb = gb3 * gb
         gc = gt3 * yukfac(1)
         gt = gt3 * yukfac(2)
         gb1 = gb13 * yukfac(7)
         gb2 = gb23 * yukfac(7)
         gt1 = gt13 * yukfac(6)
         gt2 = gt23 * yukfac(6)

         gte  = (c1susy1t3-gt3*c1sm13) * yukfac(6)
         gte2 = (c1susy2t3-gt3*c1sm23) * yukfac(6)

         c1eff0rd = gt*c1sm13 + gte
         c1eff1rd = gt*c1sm23 + gte2

      endif

      if ((dist.eq.0).and.(ptc.eq.0).and.(rapc.eq.0)) then

         write(*,105)
         write(*,*) "Calling ggh@nnlo and bbh@nnlo"
         write(*,105)

      !calling at LO
         call XSgghbbh(0,SUggh(1),SUggherr(1),SUbbh(1))

      !calling at NNLO
         call XSgghbbh(2,SUggh(3),SUggherr(3),SUbbh(3))

      !calling at NLO - PDFs also for subsequent NLO calculations
         call XSgghbbh(1,SUggh(2),SUggherr(2),SUbbh(2))
      end if

      if (norderggh.eq.0) then
         write(*,105)
         write(*,*) "Performing LO ggh calculation"
         write(*,105)
         call SUinitpdf(SUpdfs(1),SUiset)
         call checkorder(0)
      else if (norderggh.gt.0) then
         write(*,105)
         write(*,*) "Performing NLO ggh calculation"
         write(*,105)
         call SUinitpdf(SUpdfs(2),SUiset)
         call checkorder(1)
      endif

C-}}}
C-{{{ weighting of bbh results
      If (dist.eq.0) then
      !weighting bbh result for MSSM
         if ((model.eq.1).or.(model.eq.2)) then
            if (delmbfh.eq.1) then
            call bbhweights(alpha,tanb,delmb,1.D0,wAex,wLHex,wHHex)
            else
            call bbhweights(alpha,tanb,delmb,delAb,wAex,wLHex,wHHex)
            endif

            if (norderbbh.eq.0) then
               do count1=1,3
                  if (pseudo.eq.0) then
                     SUbbhsusy(count1) = wLHex(1) * SUbbh(count1) 
                     SUbbhsusyerr(count1) = wLHex(1) * SUbbherr(count1) 
                  else if (pseudo.eq.1) then
                     SUbbhsusy(count1) = wAex(1) * SUbbh(count1)
                     SUbbhsusyerr(count1) = wAex(1) * SUbbherr(count1)
                  else if (pseudo.eq.2) then
                     SUbbhsusy(count1) = wHHex(1) * SUbbh(count1)  
                     SUbbhsusyerr(count1) = wHHex(1) * SUbbherr(count1)  
                  end if
               end do
            else
               do count1=1,3
                  if (pseudo.eq.0) then
                     SUbbhsusy(count1) = wLHex(2) * SUbbh(count1) 
                     SUbbhsusyerr(count1) = wLHex(2) * SUbbherr(count1) 
                  else if (pseudo.eq.1) then
                     SUbbhsusy(count1) = wAex(2) * SUbbh(count1) 
                     SUbbhsusyerr(count1) = wAex(2) * SUbbherr(count1) 
                  else if (pseudo.eq.2) then
                     SUbbhsusy(count1) = wHHex(2) * SUbbh(count1) 
                     SUbbhsusyerr(count1) = wHHex(2) * SUbbherr(count1) 
                  end if
               end do
            end if

         end if
      end if
C-}}}
C-{{{ calculation of cross sections - NLO full contributions

      ncall = ncall1
      itmx = itmx1
      nprn = nprn1

      if (norderggh.ge.0) then
         if (model.eq.1) then
            call XSFull(SUggh,SUggherr,ggsusy,qgsusy,qqsusy,ggsusyerr,
     &qgsusyerr,qqsusyerr,CSsusy,susyerr,elw,mssm,errmssm)
         endif

         if (model.eq.0) then
         !set couplings to SM values:
            gc = yukfac(1)
            gt = yukfac(2)
            gb = yukfac(3)
            gtp = yukfac(4)
            gbp = yukfac(5)
            gb1 = 0.d0
            gb2 = 0.d0
            gt1 = 0.d0
            gt2 = 0.d0
            gte = 0.d0
            gte2 = 0.d0
            gbp1 = 0.d0
            gbp2 = 0.d0
            gbpe = 0.d0
            gbpe2 = 0.d0
            gtp1 = 0.d0
            gtp2 = 0.d0
            gtpe = 0.d0
            gtpe2 = 0.d0
            !bottom mass/Yukawa set to specified value
            !1 = mb^OS  , 2 = mb^MSbar(mb) , 3 = mb^MSbar(muR)
            !4 = mb^OS,Yb(mb^MSbar(mb)) , 5 = mb^OS,Yb(mb^MSbar(muR))
            if ((mbrunloop.eq.0).and.(mbrunyuk.eq.0)) then
               mb = mbos
               mbyuk = mbos
               dmb = 0.d0
               dgb = 0.d0
!            else if ((mbrunloop.eq.1).and.(mbrunyuk.eq.1)) then
!               mb = mbmb
!               mbyuk = mbmb
!c               dmb = (mbmb - mbos)/mbos * Pi/alphamuR
!               dmb = (mbmb - mbos)/ mb   * Pi/alphamuR
!               dgb = (mbmb - mbos)/ mbyuk * Pi/alphamuR
!               !division by mbos is not part of the derivative
!               !with respect to mb and thus appears here!
!c               dgb = 0.d0
!            else if ((mbrunloop.eq.2).and.(mbrunyuk.eq.2)) then
!               mb = mbMSbarmuR
!               mbyuk = mbMSbarmuR
!               dmb = (mbMSbarmuR - mbos)/ mb   * Pi/alphamuR
!               dgb = (mbMSbarmuR - mbos)/ mbyuk * Pi/alphamuR
            else if ((mbrunloop.eq.0).and.(mbrunyuk.eq.1)) then
               mb = mbos
               mbyuk = mbmb
               dmb = 0.d0
               gb = mbyuk/mb * yukfac(3)
               !dgb = (mbmb - mbos)/ mbyuk * Pi/alphamuR
               dgb = - cf * alphamuR / Pi * Pi / alphamuR !just 1-loop
            else if ((mbrunloop.eq.0).and.(mbrunyuk.eq.2)) then
               mb = mbos
               mbyuk = mbMSbarmuR
               dmb = 0.d0
               gb = mbyuk/mb * yukfac(3)
               !dgb = (mbMSbarmuR - mbos)/ mbyuk * Pi/alphamuR
               dgb = (- cf + 2.d0 * dlog(mbmb/muR)) 
     &         * alphamuR / Pi * Pi / alphamuR !just 1-loop
            else
               write(*,105)
               write(*,*) "Renormalization scheme for bottom"
               write(*,*) "sector not known in the SM."
               !write(*,*) "Choose both internal mass/Yukawa equal or"
               !write(*,*) "internal mass OS."
               write(*,105)
               stop
            end if
            mb2 = mb**2
            ncall = ncall1
            itmx = itmx1
            nprn = nprn1

            call XSFull(SUggh,SUggherr,ggsm,qgsm,qqsm,ggsmerr,qgsmerr,
     &           qqsmerr,CSsm,smerr,elw,sm,errsm)
         end if
      end if
C-}}}
C-{{{ showing output/writing the file-output
      call screenoutputsushi()
      call outputsushi(fhflagout,delmb)

      write(*,104) "Results written in output-file: ",jfnameact
      call printcopysushi(0,6)

 102  format('    ',i4,'  ',e16.8,'  # ',a21)
 103  format(a)
 104  format(a,a)
 105  format('#--------------------------------------------------#')
 106  format(a12,'    ',f10.5, '   GeV')
C-}}}

      end

!end of file
